<?php

namespace Tygh\Payments\Processors;

use Tygh\Registry;
use Tygh\Http;
use Tygh\Payments\RBSDiscount;

require_once realpath(dirname(__FILE__) . '/../../include.php');


class RBSPayment
{

    const ENABLE_CALLBACK = RBS_ENABLE_CALLBACK;
    const ENABLE_FISCALE_OPTIONS = RBS_ENABLE_FISCALE_OPTIONS;

    protected $module_version = '2.5.0';

    protected $_url = '';
    protected $_currency;
    protected $_login;
    protected $_password;

    protected $_two_staging = false;
    protected $_logging = false;
    protected $_send_order = false;
    protected $_tax_system = 0;
    protected $_tax_type = 0;

    protected $_FFDVersion;
    protected $_ffd_paymentMethodType;
    protected $_ffd_paymentObjectType;

    protected $_response;
    protected $_error_code = 0;
    protected $_error_text = '';

    protected $test_mode;

    public $currency_code2num = array(
        'USD' => '840',
        'UAH' => '980',
        'RON' => '946',
        'KZT' => '398',
        'KGS' => '417',
        'JPY' => '392',
        'GBR' => '826',
        'EUR' => '978',
        'CNY' => '156',
        'BYR' => '974',
        'BYN' => '933'
    );

    public function __construct($processor_data)
    {

        $this->_login = $processor_data['processor_params']['login'];
        $this->_password = $processor_data['processor_params']['password'];

        // previous versions support
        if ($processor_data['processor_params']['mode'] == 'test' || $processor_data['processor_params']['mode'] == 'dev') {
            $this->_url = RBS_TEST_URL;
            $this->test_mode = true;

        } else {
            $this->_url = RBS_PROD_URL;
            $this->test_mode = false;

            // for migrate to new payment gateway
            if (defined('RBS_PROD_URL_ALT') && defined('RBS_PROD_URL_ALT_PREFIX')) {
                if (substr($this->_login, 0, strlen(RBS_PROD_URL_ALT_PREFIX)) == RBS_PROD_URL_ALT_PREFIX) {
                    $this->_url = RBS_PROD_URL_ALT;
                } else {
                    //disable callbacks for old gateway version
                    //self::ENABLE_CALLBACK = false; //not needed in this module version
                }
            }
        }

        if (!empty($processor_data['processor_params']['two_staging'])) {
            $this->_two_staging = true;
        }

        if (!empty($processor_data['processor_params']['send_order']) && $processor_data['processor_params']['send_order'] == 'Y') {
            $this->_send_order = true;
        }

        $this->_tax_system = (!empty($processor_data['processor_params']['tax_system'])) ? $processor_data['processor_params']['tax_system'] : 0;


        if (!empty($processor_data['processor_params']['logging']) && $processor_data['processor_params']['logging'] == 'Y') {
            $this->_logging = true;
        }

        $this->_tax_type = (!empty($processor_data['processor_params']['tax_type'])) ? $processor_data['processor_params']['tax_type'] : 0;
        $this->_FFDVersion = (!empty($processor_data['processor_params']['FFDVersion'])) ? $processor_data['processor_params']['FFDVersion'] : "v1_05";
        $this->_ffd_paymentMethodType = (!empty($processor_data['processor_params']['ffd_paymentMethodType'])) ? $processor_data['processor_params']['ffd_paymentMethodType'] : 1;
        $this->_ffd_paymentObjectType = (!empty($processor_data['processor_params']['ffd_paymentObjectType'])) ? $processor_data['processor_params']['ffd_paymentObjectType'] : 1;

    }


    public function _updateGatewayCallback($login, $password, $action_address, $callback_addresses_string)
    {
        $headers = array(
            'Content-Type:application/json',
            'Authorization: Basic ' . base64_encode($login . ":" . $password)
        );
        $data['callbacks_enabled'] = true;
        $data['callback_addresses'] = $callback_addresses_string;
        $data['callback_operations'] = "deposited,approved,declinedByTimeout";

        $response = $this->_sendRBSData(json_encode($data), $action_address, $headers);
        return $response;
    }

    public function register($order_info, $protocol = 'current')
    {

        $order_id = $order_info['order_id'];
        $orderNumber = $order_id . '_' . substr(md5($order_id . TIME), 0, 3);

        //CB section update
        if (self::ENABLE_CALLBACK === true) {

            $action_adr = $this->_url;
            if ($this->test_mode == true) {
                $gate_url = str_replace("payment/rest", "mportal-uat/mvc/public/merchant/update", $action_adr);

                // "mportal-uat" error for some ONE bank
                //$gate_url = str_replace("rest", "mportal/mvc/public/merchant/update", $action_adr);

            } else {
                $gate_url = str_replace("payment/rest", "mportal/mvc/public/merchant/update", $action_adr);
            }

            $gate_url .= substr($this->_login, 0, -4); // we guess username = login w/o "-api"

            $callback_addresses_string = fn_url("payment_notification.return?payment=rbspayment&action=callback&ordernumber=$order_id", AREA, $protocol);
            if (self::ENABLE_CALLBACK !== false) {
//                $response =
                $this->_updateGatewayCallback($this->_login, $this->_password, $gate_url, $callback_addresses_string);

//                if (RBS_ENABLE_LOGGING === true) {
//                    $this->writeLog("[callback_addresses_string]: " . $callback_addresses_string . "\n[RESPONSE]: " . $response);
//                }
            }
        }

//        $order_id = $order_info['order_id'];
//        $orderNumber = $order_id . '_' . substr(md5($order_id . TIME), 0, 3);

        $rbs_args = array(
            'userName' => $this->_login,
            'password' => $this->_password,
            'orderNumber' => $orderNumber,
            'amount' => fn_format_price_by_currency($order_info['total']) * 100,
            'currency' => $this->currency_code2num[CART_SECONDARY_CURRENCY],
            'returnUrl' => fn_url("payment_notification.return?payment=rbspayment&action=return&ordernumber=$order_id", AREA, $protocol),
            'failUrl' => fn_url("payment_notification.error?payment=rbspayment&ordernumber=$order_id", AREA, $protocol),
            'jsonParams' => json_encode(
                [
                    'CMS:' => PRODUCT_NAME . " " . PRODUCT_VERSION,
                    'Module-Version: ' => $this->module_version
                ]
            ),
        );

        if (self::ENABLE_FISCALE_OPTIONS && $this->_send_order) {

            // collect $product_taxes array
            // since 2.0.3

            $product_taxes = array();

            foreach ($order_info['taxes'] as $k => $v) {
                $item_rate_value = (int)$v['rate_value'];

                foreach ($v['applies']['items']['P'] as $c => $d) {

                    if ($item_rate_value == 20) {
                        $tax_type = 6;
                    } else if ($item_rate_value == 18) {
                        $tax_type = 3;
                    } else if ($item_rate_value == 10) {
                        $tax_type = 2;
                    } else if ($item_rate_value == 0) {
                        $tax_type = 1;
                    } else {
                        $tax_type = $this->_tax_type;
                    }
                    $product_taxes[$c] = $tax_type;
                }
            }

            $rbs_args['taxSystem'] = $this->_tax_system;

            $items = array();
            $itemsCnt = 1;

            $subtotal_discount = isset($order_info['subtotal_discount']) ? $order_info['subtotal_discount'] : 0;
            $shipping_cost = isset($order_info['shipping_cost']) ? $order_info['shipping_cost'] : 0;
            $payment_surcharge = isset($order_info['payment_surcharge']) ? $order_info['payment_surcharge'] : 0;

//            $order_total = 0;

            foreach ($order_info['products'] as $value) {

                $q = isset($value['amount']) ? $value['amount'] : 1;
                $p = isset($value['price']) ? $value['price'] * 100 : 0;

                $tax_type = (!empty($product_taxes)) ? $product_taxes[$value['item_id']] : $this->_tax_type;

                $item['positionId'] = $itemsCnt++;
                $item['name'] = isset($value['product']) ? strip_tags($value['product']) : '';

                // FFDVersion
                if ($this->_FFDVersion == 'v1_05') {
                    $item['quantity'] = array(
                        'value' => $q,
                        'measure' => RBS_MEASUREMENT_NAME
                    );
                } else {
                    $item['quantity'] = array(
                        'value' => $q,
                        'measure' => RBS_MEASUREMENT_CODE
                    );
                }

                $item['itemAmount'] = $p * $q;
                $item['itemCode'] = $value['product_code'] . $item['positionId'];
                $item['tax'] = array(
                    'taxType' => $tax_type
                );
                $item['itemPrice'] = $p;


                $attributes = array();
                $attributes[] = array(
                    "name" => "paymentMethod",
                    "value" => $this->_ffd_paymentMethodType
                );
                $attributes[] = array(
                    "name" => "paymentObject",
                    "value" => $this->_ffd_paymentObjectType
                );
                $item['itemAttributes']['attributes'] = $attributes;


                $items[] = $item;
            }

//             DISCOUNT_VALUE_SECTION
//            if ($subtotal_discount > 0) {
//                $new_order_total = 0;
//                foreach ($items as &$i) {
//                    $p_discount = round($i['itemAmount'] / $subtotal_discount, 2) * 100;
//                    self::correctBundleItem($i, $p_discount);
////                    $i['discount']['discountType'] = 'summ';
////                    $i['discount']['discountValue'] += $p_discount;
//                    $new_order_total += $i['itemAmount'];
//                }
//                $rbs_args['amount'] = $new_order_total;
//            }

            // PAYMENT_SURCHARGE
            if ($payment_surcharge > 0) {
                $itemSurcharge['positionId'] = $itemsCnt++;
                $itemSurcharge['name'] = !empty($order_info['payment_method']['surcharge_title']) ? $order_info['payment_method']['surcharge_title'] : __("addons.rbspayment.payment_surcharge");

                if ($this->_FFDVersion == 'v1_05') {
                    $itemSurcharge['quantity'] = array(
                        'value' => 1,
                        'measure' => RBS_MEASUREMENT_NAME
                    );
                } else {
                    $itemSurcharge['quantity'] = array(
                        'value' => 1,
                        'measure' => RBS_MEASUREMENT_CODE
                    );
                }

                $itemSurcharge['itemAmount'] = $itemSurcharge['itemPrice'] = $payment_surcharge * 100;
                $itemSurcharge['itemCode'] = 'Surcharge';
                $itemSurcharge['tax'] = array(
                    'taxType' => $tax_type
                );

                // FFD 1.05 added
//                if ($this->_FFDVersion == 'v1_05') {
                $attributes = array();
                $attributes[] = array(
                    "name" => "paymentMethod",
                    "value" => $this->_ffd_paymentMethodType
                );
                $attributes[] = array(
                    "name" => "paymentObject",
                    "value" => 4
                );
                $itemSurcharge['itemAttributes']['attributes'] = $attributes;
//                }
                $items[] = $itemSurcharge;
            }

            // DELIVERY
            if ($shipping_cost > 0) {
                $itemShipment['positionId'] = $itemsCnt;
                $itemShipment['name'] = __("addons.rbspayment.delivery");

                if ($this->_FFDVersion == 'v1_05') {
                    $itemShipment['quantity'] = array(
                        'value' => 1,
                        'measure' => RBS_MEASUREMENT_NAME
                    );
                } else {
                    $itemShipment['quantity'] = array(
                        'value' => 1,
                        'measure' => RBS_MEASUREMENT_CODE
                    );
                }

                $itemShipment['itemAmount'] = $itemShipment['itemPrice'] = $shipping_cost * 100;
                $itemShipment['itemCode'] = 'Delivery';
                $itemShipment['tax'] = array(
                    'taxType' => $tax_type
                );

                // FFD 1.05 added
//                if ($this->_FFDVersion == 'v1_05') {
                $attributes = array();
                $attributes[] = array(
                    "name" => "paymentMethod",
                    "value" => $this->_ffd_paymentMethodType
                );
                $attributes[] = array(
                    "name" => "paymentObject",
                    "value" => 4
                );
                $itemShipment['itemAttributes']['attributes'] = $attributes;
//                }
                $items[] = $itemShipment;
            }

            $order_bundle = array(
                'orderCreationDate' => time(),
                'customerDetails' => array(
                    'email' => $order_info['email'],
                    'phone' => preg_replace('/\D+/', '', $order_info['phone'])
                ),
                'cartItems' => array('items' => $items)
            );

            // DISCOUNT CALCULATE
            $discountHelper = new RBSDiscount();

            $discount = $discountHelper->discoverDiscount($rbs_args['amount'], $order_bundle['cartItems']['items']);
            if ($discount > 0) {
                $discountHelper->setOrderDiscount($discount);
                $recalculatedPositions = $discountHelper->normalizeItems($order_bundle['cartItems']['items']);
//                $recalculatedAmount = $this->rbs->discountHelper->getResultAmount();
                $order_bundle['cartItems']['items'] = $recalculatedPositions;
            }

            $rbs_args['orderBundle'] = json_encode($order_bundle);

        }

        $action_adr = 'register.do';
        if ($this->_two_staging) {
            $action_adr = 'registerPreAuth.do';
        }

        $this->_response = $this->_sendRBSData(http_build_query($rbs_args, '', '&'), $this->_url . $action_adr);

        if ($this->_logging) {
            $rbs_args['password'] = '**removed from log**';
            self::_writeRBSLog($rbs_args, 'Register Order');
        }

        $this->_response = json_decode($this->_response, true);

        if (!empty($this->_response['errorCode'])) {
            $this->_error_code = $this->_response['errorCode'];
            $this->_error_text = $this->_response['errorMessage'];
        }

        return $this->_response;
    }

    public function _sendRBSData($data, $action_address, $headers = array())
    {

        $ch = curl_init();
        curl_setopt_array($ch, array(
            CURLOPT_HTTPHEADER => $headers,
            CURLOPT_VERBOSE => true,
            CURLOPT_SSL_VERIFYHOST => false,
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_URL => $action_address,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POST => true,
            CURLOPT_POSTFIELDS => $data,
            //                CURLOPT_ENCODING, "gzip",
        ));
        $response = curl_exec($ch);
        curl_close($ch);
        return $response;
    }

    public function correctBundleItem(&$item, $discount)
    {

        $item['itemAmount'] -= $discount;
        $item['itemPrice'] = $item['itemAmount'] % $item['quantity']['value'];
        if ($item['itemPrice'] != 0) {
            $item['itemAmount'] += $item['quantity']['value'] - $item['itemPrice'];
        };

        $item['itemPrice'] = $item['itemAmount'] / $item['quantity']['value'];

        return $item;
    }

    public function getOrder($transaction_id)
    {
        $data = array(
            'userName' => $this->_login,
            'password' => $this->_password,
            'orderId' => $transaction_id
        );

//        $this->_response = Http::post($this->_url . 'getOrderStatus.do', $data);
        $data = http_build_query($data, '', '&');
        $this->_response = $this->_sendRBSData($data, $this->_url . 'getOrderStatus.do');


        $this->_response = json_decode($this->_response, true);

        if (!empty($this->_response['errorCode'])) {
            $this->_error_code = $this->_response['errorCode'];
            $this->_error_text = $this->_response['errorMessage'];
        }

        return $this->_response;
    }

    public function getOrderExtended($transaction_id)
    {
        $data = array(
            'userName' => $this->_login,
            'password' => $this->_password,
            'orderId' => $transaction_id
        );

//        $this->_response = Http::post($this->_url . 'getOrderStatusExtended.do', $data);
        $data = http_build_query($data, '', '&');
        $this->_response = $this->_sendRBSData($data, $this->_url . 'getOrderStatusExtended.do');


        $this->_response = json_decode($this->_response, true);

        if (!empty($this->_response['errorCode'])) {
            $this->_error_code = $this->_response['errorCode'];
            $this->_error_text = $this->_response['errorMessage'];
        }

        return $this->_response;
    }

    public function getErrorCode()
    {
        return $this->_error_code;
    }

    public function getErrorText()
    {
        return $this->_error_text;
    }

    public function isError()
    {
        return !empty($this->_error_code);
    }

    function _writeRBSLog($var, $info = true)
    {
        $information = "";
        if ($var) {
            if ($info) {
                $information = "\n\n";
                $information .= str_repeat("-=", 64);
                $information .= "\nDate: " . date('Y-m-d H:i:s');
            }

            $result = $var;
            if (is_array($var) || is_object($var)) {
                $result = "\n" . print_r($var, true);
            }
            $result .= "\n\n";
            $path = dirname(__FILE__) . '/cs_rbspayment_' . date('Y-m') . '.log';
            error_log($information . $result, 3, $path);
            return true;
        }
        return false;
    }


    /**
     * @param $data
     * @param $title
     */
    public static function writeLog($data, $title)
    {

        $path = fn_get_files_dir_path();
        fn_mkdir($path);

        $file_name = '/cs_rbspayment_' . date('Y-m') . '.log';
        $file = fopen($path . $file_name, 'a');

        if (!empty($file)) {
            fputs($file, 'TIME: ' . date('Y-m-d H:i:s', TIME) . " > [" . $title . "]: \n");
            fputs($file, fn_array2code_string($data) . "\n\n");
            fclose($file);
        }
    }

    public static function convertSum($price)
    {
//        if (CART_PRIMARY_CURRENCY != CART_SECONDARY_CURRENCY) {
        $price = fn_format_price_by_currency($price, CART_PRIMARY_CURRENCY, CART_SECONDARY_CURRENCY);
//        }

        $price = fn_format_rate_value($price, 'F', 2, '.', '', '');

        return $price;
    }
}